@@ -19,7 +19,7 @@ Environment Variables
   port:           Apache Web Server Port. (Default: 80, SSL: 443)
   user:           User in case authentication is required for access to 
                   server-status page.
-  password:       User in case authentication is required for access 
+  password:       Password in case authentication is required for access 
                   to server-status page.
   statuspath:     Path for Apache Web Server Status Page.
                   (Default: server-status)
@@ -34,8 +34,8 @@ Environment Variables
 
 """
 # Munin  - Magic Markers
-#%# family=manual
-#%# capabilities=noautoconf nosuggest
+#%# family=auto
+#%# capabilities=autoconf nosuggest
 
 import sys
 from pymunin import MuninGraph, MuninPlugin, muninMain
@@ -69,7 +69,7 @@ class MuninApachePlugin(MuninPlugin):
         MuninPlugin.__init__(self, argv, env, debug)
         
         self._host = self.envGet('host')
-        self._port = self.envGet('port')
+        self._port = self.envGet('port', None, int)
         self._user = self.envGet('user')
         self._password = self.envGet('password')
         self._statuspath = self.envGet('statuspath')
@@ -120,6 +120,17 @@ class MuninApachePlugin(MuninPlugin):
             self.setGraphVal('apache_workers', 'busy', stats['BusyWorkers'])
             self.setGraphVal('apache_workers', 'idle', stats['IdleWorkers'])
             self.setGraphVal('apache_workers', 'max', stats['MaxWorkers'])
+            
+    def autoconf(self):
+        """Implements Munin Plugin Auto-Configuration Option.
+        
+        @return: True if plugin can be  auto-configured, False otherwise.
+                 
+        """
+        apacheInfo = ApacheInfo(self._host, self._port,
+                                self._user, self._password, 
+                                self._statuspath, self._ssl)
+        return apacheInfo is not None
 
 
 def main():
